'===============
'NumberTouch.bas v1.1
'===============
'A 'Numberhood' clone in QB64 (math puzzle game).
'Coded by Dav AUG/2023

'New for v1.1:  - Board auto-sizes to fit users desktop
'               - Fixed buggy mouse code
'               - Added some game sounds

'====
'GOAL:
'====

'Put numbered blocks together to turn them green.
'When all blocks turn green, level is completed.

'===========
'HOW TO PLAY:
'===========

'First Select a block to move to an empty spot.
'Then select the spot to move it to. The numbers in the
'block say how many other blocks should be touching it.
'When the correct number is touching it, it will
'turn green (ie: 1 means only one block should touch it)
'If too many are touching it, it will turn red.
'Numbers with yellow border can't be moved (level 8 & up).
'Turn all blocks on board green to clear the level.

'There are 15 levels. Left/Right arrows navigate levels.
'Current levels is displayed in the title bar.
'Press ESC to quit quickly at any time.


dh = FIX(_DESKTOPHEIGHT * .85)

SCREEN _NEWIMAGE(dh, dh, 32)
DO: LOOP UNTIL _SCREENEXISTS

CLS , _RGB(77, 77, 77)

level = 1
maxlevel = 15

'======
restart:
'======

_SNDPLAYFILE ("sfx_magic.ogg")

'Show current leven in title bar
titl$ = "NumberTouch:" + STR$(level) + " of" + STR$(maxlevel)
_TITLE titl$

'define board size, and tile size
REDIM SHARED grid, size
grid = 9
size = dh / grid

REDIM SHARED bm(grid * grid) 'Marker for non-movable blocks
REDIM SHARED bx(grid * grid), by(grid * grid) 'x/y position on blocks
REDIM SHARED bv$(grid * grid) 'Block data (value)
REDIM SHARED fc(grid * grid) 'Color of Block (red,green,normal)
REDIM SHARED bcon$(grid * grid) 'Block connection data

'Use this alot, so define it.
REDIM SHARED block$: block$ = "black" 'black block

'init default values, and generate x/y values...
bc = 1
FOR r = 1 TO grid
    FOR c = 1 TO grid
        x = (r * size): y = (c * size)
        'draw board of black blocks
        ttmp = _LOADIMAGE(block$ + ".png")
        _PUTIMAGE (x - size, y - size)-(x, y), ttmp
        _FREEIMAGE ttmp
        bx(bc) = x - size: by(bc) = y - size 'button x/y position
        fc(bc) = 0 'fill color of button
        bv$(bc) = block$ 'fill values with blocks
        bcon$(bc) = "udlr" 'block connections
        bm(bc) = 0
        bc = bc + 1
    NEXT c
NEXT r


'define connections for each block (left,right,up,down)
'and adjust connections of corner buttons
bcon$(1) = "rd": bcon$(grid) = "rd"
bcon$((grid * grid) - (grid - 1)) = "ld"
bcon$(grid * grid) = "ul"
'left side
FOR s = 2 TO grid - 1
    bcon$(s) = "udr"
NEXT s
'right side
FOR s = ((grid * grid) - (grid - 2)) TO (grid * grid - 1)
    bcon$(s) = "udl"
NEXT s

'load level
GOSUB setlevel


'===
main: 'Main level playing loop
'===

selected = 0 'No selection made yet

GOSUB updateboard 'draw board

'Show help popup for certain levels
IF level = 1 THEN help$ = "help1.png"
IF level = 2 THEN help$ = "help2.png"
IF level = 3 THEN help$ = "help3.png"
IF level = 4 THEN help$ = "help4.png"
IF level = 8 THEN help$ = "help8.png"
IF level = 1 OR level = 2 OR level = 3 OR level = 4 OR level = 8 THEN
    ttmp = _LOADIMAGE(help$)
    _PUTIMAGE (15, 15), ttmp
    _FREEIMAGE ttmp
END IF

second: 'Go back here when making second choice

DO

    'wait until mouse button up to continue
    WHILE _MOUSEBUTTON(1) <> 0: n = _MOUSEINPUT: WEND

    WHILE _MOUSEINPUT: WEND 'mouse data

    'If user clicked mouse
    IF _MOUSEBUTTON(1) THEN
        'see where they clicked
        mx = _MOUSEX: my = _MOUSEY

        'Check blocks...
        FOR t = 1 TO (grid * grid)
            'Block loction...
            tx = bx(t): tx2 = bx(t) + size
            ty = by(t): ty2 = by(t) + size

            'Only do a number block
            IF bv$(t) <> block$ THEN
                'determine where they clicked
                IF mx >= tx AND mx <= tx2 THEN
                    IF my >= ty AND my <= ty2 THEN

                        'only handle non movable numbers
                        IF bm(t) = 0 THEN

                            'if first choice...
                            IF selected = 0 THEN
                                'only select a number block (no space)
                                IF bv$(t) <> "" THEN

                                    selected = 1

                                    'Now highlight block
                                    nam$ = swapit$(t) + "-sel.png"
                                    ttmp = _LOADIMAGE(nam$)
                                    _PUTIMAGE (bx(t), by(t))-(bx(t) + size, by(t) + size), ttmp
                                    _FREEIMAGE ttmp
                                    _SNDPLAYFILE ("sfx_pop1.wav")

                                    selbtn = t 'save picked block number
                                    GOTO second 'now get second choice

                                END IF

                            ELSE 'making second choice
                                'only move to blank space
                                IF bv$(t) = "" THEN
                                    bv$(t) = bv$(selbtn): bv$(selbtn) = ""
                                    _SNDPLAYFILE ("sfx_pop2.wav")
                                    GOTO main
                                ELSE
                                    'clear previous highlighted selection
                                    selected = 0: GOSUB updateboard: selected = 1

                                    'highlight this block
                                    nam$ = swapit$(t) + "-sel.png"
                                    ttmp = _LOADIMAGE(nam$)
                                    _PUTIMAGE (bx(t), by(t))-(bx(t) + size, by(t) + size), ttmp
                                    _FREEIMAGE ttmp
                                    selbtn = t
                                    _SNDPLAYFILE ("sfx_pop1.wav")
                                    GOTO second
                                END IF
                            END IF
                        END IF
                    END IF
                END IF
            END IF

        NEXT t

    END IF

    'Check for a keypress
    kk$ = INKEY$
    'Right arrows advance to next level
    IF kk$ = CHR$(0) + CHR$(77) THEN
        level = level + 1
        IF level > maxlevel THEN level = 1
        GOTO restart
    END IF
    'Left Arrows go back a level
    IF kk$ = CHR$(0) + CHR$(75) THEN
        level = level - 1
        IF level < 1 THEN level = 15
        GOTO restart
    END IF

LOOP UNTIL kk$ = CHR$(27) 'ESC key quits game, closes window

SYSTEM


'==========
updateboard: 'Draws board based on saved data in arrays
'==========

FOR s = 1 TO grid * grid

    'only update non block buttons
    IF bv$(s) <> block$ THEN

        'define color for each block (red,green, normal)
        'based on how many blocks are touching it.

        'Empty spaces are ignored
        IF bv$(s) = "" THEN
            fc(s) = 0
        ELSE
            num = 0 'block default color if normal (1)
            fc(s) = 1

            'look left, if block next door add a count
            IF INSTR(bcon$(s), "l") THEN
                peekypoo = 1
                IF bv$(s - grid) = block$ THEN peekypoo = 0
                IF bv$(s - grid) = "" THEN peekypoo = 0
                IF peekypoo = 1 THEN num = num + 1
            END IF
            'look right
            IF INSTR(bcon$(s), "r") THEN
                peekypoo = 1
                IF bv$(s + grid) = block$ THEN peekypoo = 0
                IF bv$(s + grid) = "" THEN peekypoo = 0
                IF peekypoo = 1 THEN num = num + 1
            END IF
            'look up
            IF INSTR(bcon$(s), "u") THEN
                peekypoo = 1
                IF bv$(s - 1) = block$ THEN peekypoo = 0
                IF bv$(s - 1) = "" THEN peekypoo = 0
                IF peekypoo = 1 THEN num = num + 1
            END IF
            'look down
            IF INSTR(bcon$(s), "u") THEN
                peekypoo = 1
                IF bv$(s + 1) = block$ THEN peekypoo = 0
                IF bv$(s + 1) = "" THEN peekypoo = 0
                IF peekypoo = 1 THEN num = num + 1
            END IF

            'Assign color value based of num tally
            IF num = VAL(bv$(s)) THEN fc(s) = 3 'green
            IF num > VAL(bv$(s)) THEN fc(s) = 2 'red

        END IF

        'finally draw the block based on color assigned
        IF fc(s) = 0 THEN tag$ = ".png" 'blank, white
        IF fc(s) = 1 THEN tag$ = "-nrm.png" 'normal
        IF fc(s) = 2 THEN tag$ = "-red.png" 'red
        IF fc(s) = 3 THEN tag$ = "-grn.png" 'green
        nam$ = swapit$(s) + tag$
        ttmp = _LOADIMAGE(nam$)
        _PUTIMAGE (bx(s), by(s))-(bx(s) + size, by(s) + size), ttmp
        _FREEIMAGE ttmp

    END IF
NEXT s

'check if all are green
done = 1 'default is done
FOR s = 1 TO grid * grid
    'only check number buttons
    IF bv$(s) <> block$ THEN
        IF bv$(s) <> "" THEN
            IF fc(s) <> 3 THEN
                done = 0: EXIT FOR
            END IF
        END IF
    END IF
NEXT s

'If level done, advance level
IF done = 1 THEN
    level = level + 1
    _SNDPLAYFILE ("sfx_clap.ogg")
    FOR s = 100 TO 3000 STEP 400
        'SOUND s, .5: SOUND 0, .2
        _DELAY .01
    NEXT s

    _DELAY 1

    'change pieces to check mark
    FOR i = 1 TO grid * grid
        IF bv$(i) = block$ THEN GOTO skipp
        IF bv$(i) = "" THEN GOTO skipp
        'Change piece to checkmark
        ttmp = _LOADIMAGE("check.png")
        _PUTIMAGE (bx(i), by(i))-(bx(i) + size, by(i) + size), ttmp
        _FREEIMAGE ttmp
        _DELAY .05
        skipp:
    NEXT i

    _DELAY 1.5

    IF level > maxlevel THEN level = 1 'all levels done? Restart...

    GOTO restart

END IF

RETURN

'====================================================================
setlevel:
'========

IF level = 1 THEN
    bv$(31) = "1": bv$(32) = "": bv$(33) = "": bv$(40) = "": bv$(41) = ""
    bv$(42) = "": bv$(49) = "": bv$(50) = "": bv$(51) = "1"
END IF

IF level = 2 THEN
    bv$(31) = "": bv$(32) = "": bv$(33) = "1": bv$(40) = "": bv$(41) = "2"
    bv$(42) = "": bv$(49) = "1": bv$(50) = "": bv$(51) = ""
END IF

IF level = 3 THEN
    bv$(31) = "1": bv$(32) = "2": bv$(33) = "": bv$(40) = "1": bv$(41) = "1"
    bv$(42) = "3": bv$(49) = "": bv$(50) = "": bv$(51) = ""
END IF

IF level = 4 THEN
    bv$(31) = "1": bv$(32) = "0": bv$(33) = "": bv$(40) = "": bv$(41) = "2"
    bv$(42) = "": bv$(49) = "3": bv$(50) = "2": bv$(51) = "2"
END IF

IF level = 5 THEN
    bv$(30) = "3": bv$(31) = "": bv$(32) = "": bv$(33) = "2": bv$(34) = "1"
    bv$(39) = "": bv$(40) = "": bv$(41) = "": bv$(42) = "2": bv$(43) = "1"
    bv$(48) = "4": bv$(49) = "": bv$(50) = "": bv$(51) = "2": bv$(52) = "1"
END IF

IF level = 6 THEN
    bv$(23) = "1": bv$(31) = "2": bv$(32) = "": bv$(33) = "2": bv$(39) = "4":
    bv$(40) = "1": bv$(41) = "": bv$(42) = "1": bv$(43) = "": bv$(49) = "1":
    bv$(50) = "2": bv$(51) = "": bv$(59) = "2":
END IF

IF level = 7 THEN
    bv$(14) = "": bv$(23) = "2": bv$(32) = "": bv$(33) = "1": bv$(34) = "2":
    bv$(41) = "2": bv$(42) = "2": bv$(43) = "1": bv$(50) = "": bv$(51) = "1":
    bv$(52) = "3": bv$(61) = "": bv$(70) = "":
END IF

IF level = 8 THEN
    bv$(25) = "": bv$(26) = "2": bv$(31) = "": bv$(32) = "1": bv$(34) = "2":
    bv$(35) = "": bv$(38) = "": bv$(39) = "2": bv$(40) = "3": bm(40) = 1:
    bv$(41) = "2": bv$(42) = "1": bv$(43) = "4": bm(43) = 1: bv$(44) = "":
    bv$(49) = "": bv$(50) = "1": bv$(52) = "1": bv$(53) = "1": bv$(61) = "":
    bv$(62) = "":
END IF

IF level = 9 THEN
    bv$(21) = "": bv$(22) = "": bv$(23) = "": bv$(24) = "2": bv$(25) = "2":
    bv$(29) = "": bv$(30) = "4": bm(30) = 1: bv$(31) = "": bv$(32) = "":
    bv$(33) = "2": bv$(34) = "1": bv$(38) = "": bv$(39) = "": bv$(40) = "":
    bv$(41) = "": bv$(42) = "3": bv$(43) = "1": bv$(47) = "": bv$(48) = "2":
    bm(48) = 1:
    bv$(49) = "3": bm(49) = 1: bv$(50) = "": bv$(51) = "": bv$(52) = "1":
    bv$(57) = "": bv$(58) = "": bv$(59) = "": bv$(60) = "": bv$(61) = "1":
END IF

IF level = 10 THEN
    bv$(13) = "": bv$(14) = "1": bv$(15) = "0": bv$(21) = "": bv$(22) = "0":
    bv$(23) = "2": bm(23) = 1: bv$(24) = "": bv$(25) = "": bv$(30) = "0":
    bv$(31) = "1": bv$(32) = "": bv$(33) = "": bv$(34) = "1": bv$(39) = "":
    bv$(40) = "0": bv$(41) = "1": bm(41) = 1: bv$(42) = "0": bv$(43) = "":
    bv$(48) = "": bv$(49) = "": bv$(50) = "0": bv$(51) = "": bv$(52) = "1":
    bv$(57) = "1": bv$(58) = "": bv$(59) = "2": bm(59) = 1: bv$(60) = "":
    bv$(61) = "": bv$(67) = "": bv$(68) = "": bv$(69) = "0":

END IF

IF level = 11 THEN
    bv$(20) = "1": bv$(22) = "": bv$(23) = "": bv$(24) = "": bv$(25) = "":
    bv$(29) = "2": bv$(30) = "": bv$(31) = "0": bm(31) = 1: bv$(32) = "":
    bv$(33) = "3": bm(33) = 1: bv$(34) = "": bv$(35) = "3": bv$(38) = "2":
    bv$(39) = "": bv$(40) = "": bv$(41) = "2": bm(41) = 1: bv$(42) = "":
    bv$(43) = "": bv$(44) = "3": bv$(47) = "2": bv$(48) = "": bv$(49) = "1"
    bm(49) = 1: bv$(50) = "": bv$(51) = "4": bm(51) = 1: bv$(52) = "": bv$(53) = "4":
    bv$(56) = "1": bv$(58) = "": bv$(59) = "": bv$(60) = "": bv$(61) = "":

END IF

IF level = 12 THEN
    bv$(26) = "2": bv$(29) = "1": bv$(30) = "": bv$(31) = "2": bv$(32) = "":
    bv$(33) = "1": bv$(34) = "2": bv$(35) = "": bv$(38) = "2": bv$(39) = "3": bm(39) = 1:
    bv$(40) = "1": bv$(41) = "4": bm(41) = 1: bv$(42) = "2": bv$(43) = "4": bm(43) = 1:
    bv$(44) = "1": bv$(47) = "": bv$(48) = "": bv$(49) = "1": bv$(50) = "2":
    bv$(51) = "3": bv$(52) = "1": bv$(53) = "": bv$(59) = "":
END IF

IF level = 13 THEN
    bv$(22) = "": bv$(23) = "0": bv$(24) = "0": bv$(25) = "0": bv$(30) = "":
    bv$(31) = "": bv$(32) = "1": bv$(33) = "1": bv$(34) = "1": bv$(39) = "":
    bv$(40) = "": bv$(41) = "": bv$(42) = "": bv$(43) = "": bv$(48) = "":
    bv$(49) = "": bv$(50) = "2": bv$(51) = "2": bv$(52) = "2": bv$(58) = "":
    bv$(59) = "3": bv$(60) = "3": bv$(61) = "3":

END IF

IF level = 14 THEN
    bv$(11) = "1": bv$(12) = "": bv$(13) = "": bv$(14) = "": bv$(15) = "":
    bv$(16) = "": bv$(20) = "1": bv$(21) = "0": bm(21) = 1: bv$(22) = "":
    bv$(23) = "2": bm(23) = 1: bv$(24) = "": bv$(25) = "0": bv$(29) = "1":
    bv$(30) = "": bv$(31) = "0": bm(31) = 1: bv$(32) = "": bv$(33) = "0":
    bv$(34) = "": bv$(38) = "1": bv$(39) = "0": bm(39) = 1: bv$(40) = "":
    bv$(41) = "1": bm(41) = 1: bv$(42) = "": bv$(43) = "3": bm(43) = 1:
    bv$(47) = "1": bv$(48) = "": bv$(49) = "0": bm(49) = 1: bv$(50) = "":
    bv$(51) = "0": bv$(52) = "": bv$(56) = "1": bv$(57) = "0": bm(57) = 1:
    bv$(58) = "": bv$(59) = "2": bm(59) = 1: bv$(60) = "": bv$(61) = "0":
    bv$(65) = "2": bv$(66) = "": bv$(67) = "": bv$(68) = "": bv$(69) = "":
    bv$(70) = "":
END IF

IF level = 15 THEN
    bv$(12) = "1": bv$(13) = "1": bv$(14) = "1": bv$(15) = "1": bv$(16) = "1":
    bv$(20) = "1": bv$(21) = "": bv$(22) = "": bv$(23) = "": bv$(24) = "":
    bv$(25) = "": bv$(26) = "1": bv$(29) = "2": bv$(30) = "": bv$(31) = "0": bm(31) = 1:
    bv$(32) = "": bv$(33) = "0": bm(33) = 1: bv$(34) = "": bv$(35) = "1":
    bv$(38) = "2": bv$(39) = "": bv$(40) = "": bv$(41) = "0": bm(41) = 1:
    bv$(42) = "": bv$(43) = "": bv$(44) = "1": bv$(47) = "2": bv$(48) = "":
    bv$(49) = "0": bm(49) = 1: bv$(50) = "": bv$(51) = "0": bm(51) = 1:
    bv$(52) = "": bv$(53) = "1": bv$(56) = "2": bv$(57) = "": bv$(58) = "":
    bv$(59) = "": bv$(60) = "": bv$(61) = "": bv$(62) = "1": bv$(66) = "1":
    bv$(67) = "1": bv$(68) = "1": bv$(69) = "1": bv$(70) = "1":
END IF

RETURN

'===================
FUNCTION swapit$ (n)
    'swaps out file name based on if number block is movable or not

    a$ = ""
    IF bv$(n) = "" THEN a$ = "white"

    IF bv$(n) = "0" THEN
        a$ = "0"
        IF bm(n) = 1 THEN a$ = "0m"
    END IF
    IF bv$(n) = "1" THEN
        a$ = "1"
        IF bm(n) = 1 THEN a$ = "1m"
    END IF
    IF bv$(n) = "2" THEN
        a$ = "2"
        IF bm(n) = 1 THEN a$ = "2m"
    END IF
    IF bv$(n) = "3" THEN
        a$ = "3"
        IF bm(n) = 1 THEN a$ = "3m"
    END IF
    IF bv$(n) = "4" THEN
        a$ = "4"
        IF bm(n) = 1 THEN a$ = "4m"
    END IF
    swapit$ = a$
END FUNCTION

